find_program(AVR_CC avr-gcc)
find_program(AVR_CXX avr-g++)
find_program(AVR_OBJCOPY avr-objcopy)
find_program(AVR_SIZE_TOOL avr-size)
find_program(AVRDUDE avrdude)


function(add_avr_executable EXECUTABLE_NAME MCU)

    set(elf_file ${EXECUTABLE_NAME}-${MCU}.elf)
    set(hex_file ${EXECUTABLE_NAME}-${MCU}.hex)
    set(map_file ${EXECUTABLE_NAME}-${MCU}.map)
    add_executable(${elf_file} EXCLUDE_FROM_ALL ${ARGN})
    set(common_opts "-mmcu=${MCU} -fshort-enums -fpack-struct")

    set_target_properties(
        ${elf_file}
        PROPERTIES
            COMPILE_FLAGS "${common_opts}"
            LINK_FLAGS "${common_opts} -Wl,-Map,${map_file}"
    )

    add_custom_command(
        OUTPUT ${hex_file}
        COMMAND
            ${AVR_OBJCOPY} -j .text -j .data -O ihex ${elf_file} ${hex_file}
        COMMAND
            ${AVR_SIZE_TOOL} ${elf_file}
        DEPENDS ${elf_file}
    )
    list(APPEND all_hex_files ${hex_file})
    list(APPEND all_map_files ${map_file})


    add_custom_target(
        ${EXECUTABLE_NAME}
        ALL
        DEPENDS ${all_hex_files}
    )
    get_directory_property(clean_files ADDITIONAL_MAKE_CLEAN_FILES)
    list(APPEND clean_files ${all_map_files})
    set_directory_properties(
        PROPERTIES
        ADDITIONAL_MAKE_CLEAN_FILES "${clean_files}"
    )

    add_custom_target(
        upload_${EXECUTABLE_NAME}
        ${AVRDUDE} -p ${MCU} -c ${AVR_PROGRAMMER} ${AVRDUDE_OPTIONS}
            -U flash:w:${hex_file}
            -P ${AVR_PROGRAMMER_DEVICE}
        DEPENDS ${hex_file}
        COMMENT "Uploading using programmer ${AVR_PROGRAMMER}"
    )
endfunction(add_avr_executable)

