#include "lcd.h"

static inline void _lcd_delay_short()
{
    _delay_us( 30 );
}

static inline void _lcd_delay_long()
{
    _delay_ms( 2 );
}

static inline void _lcd_command_mode( uint8_t cmd )
{
    if( cmd )
        SPI_PORT &= ~( 1 << SPI_RS ); /* Set SPI_RS to low */
    else
        SPI_PORT |= ( 1 << SPI_RS );  /* Set SPI_RS to high */
}

static inline void _lcd_reset_display()
{
	/* Release display from reset */
	_delay_ms( 10 );
	RESET_DDR  |= ( 1 << RESET_PIN );
	RESET_PORT |= ( 1 << RESET_PIN );
}

/* 5V initialization sequence for EA DOGM163 */
/* according to EA DOGM data sheet, p. 7     */
void lcd_init()
{
	SPI_DDR |= ( 1 << SPI_RS ) | ( 1 << SPI_MOSI ) | ( 1 << SPI_SCK ) | ( 1 << SPI_SS );
    spi_init();

    _lcd_reset_display();
	_delay_ms( 50 ); /* Wait until display power supply is stable */

	_lcd_command_mode( true );
	spi_send( 0b00111001 ); /* Function Set: 8bit, 2 lines, instr. table 1 */
	_lcd_delay_short();

	spi_send(0b00010101); /* Bias Set: 1/5, fixed on high (3 line) */
	_lcd_delay_short();

	spi_send(0b01010000); /* Power Contr.: ICON and Booster off, !contrast */
	_lcd_delay_short();

	spi_send(0b01101100); /* Follower Contr.: Voltage follower and gain */
	_lcd_delay_short();

	spi_send(0b01111101); /* Contrast Set: Set contrast */
	_lcd_delay_short();

	spi_send(0b00111000); /* Function Set: Switch back to instr. table 0 */
	_lcd_delay_short();

	spi_send(0b00001100); /* Display ON: Cursor underscore and blinking */
	_lcd_delay_short();

	spi_send(0b00000001); /* Clear Display: Clear display, cursor to home */
	_lcd_delay_long();

	spi_send(0b00000110); /* Entry Mode Set: Cursor auto-increment */
	_lcd_delay_short();
}

/* Clear display and set cursor to home */
void lcd_clear_home()
{
	_lcd_command_mode( true );
	spi_send( 0b00000001 );
	_lcd_delay_long();
}

/* Set cursor to a specific location */
/* 0..15 -> Row 1, 16..31 -> Row 2, 32..47 -> Row 3 */
void lcd_set_cursor(uint8_t pos)
{
	_lcd_command_mode( true );
	spi_send( 0b10000000 | ( pos & 0b01111111 ) );
	_lcd_delay_short();
}

void lcd_showchar( char c )
{
    _lcd_command_mode( false );
    spi_send( c );
    _lcd_delay_short();
}

/* Output character at current cursor position */
int lcd_putchar(char c, FILE* unused)
{
    lcd_showchar( c );
	return 0;
}

