#include "spi.h"

/* Initialize SPI */
void spi_init()
{
	SPCR = ( 0 << SPIE ) | /* SPI interrupt disable */
	       ( 1 << SPE  ) | /* SPI enable */
	       ( 0 << DORD ) | /* Data order: MSB first */
	       ( 1 << MSTR ) | /* Master mode */
	       ( 0 << CPOL ) | /* Clock polarity: CTRL_SCK low when idle */
	       ( 0 << CPHA ) | /* Clock phase: sample on rising CTRL_SCK edge */
	       ( 0 << SPR1 ) | /* Clock frequency: f_OSC/8 => 2Mbit with SPI2X=0 */
	       ( 0 << SPR0 );
	SPSR = 0; /* Do not use doubled clock frequency */
}

/* Send a character over SPI and wait for transmission to complete */
void spi_send( uint8_t data )
{
	static volatile uint8_t foo;
	/* Schedule character for transmission */
	SPDR = data;

	/* Wait for transmission to complete */
	while ( !( SPSR & ( 1 << SPIF ) ) )
        ;

	/* Dummy read of SPDR to reset SPIF */
	foo = SPDR;
}
